#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager

//for Wifi Indicator LED status
#include <Ticker.h>
Ticker ticker;

//Library to Control WS2812B
#include <Adafruit_NeoPixel.h>
//Configuration for LED
// Which pin on the Arduino is connected to the NeoPixels?
#define LED_PIN     15
// How many NeoPixels are attached to the Arduino?
#define LED_COUNT  1
// NeoPixel brightness, 0 (min) to 255 (max)
#define BRIGHTNESS 50
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRBW + NEO_KHZ400);

// Configuration for Web server
// Set web server port number to 80
WiFiServer server(80);

// Variable to store the HTTP request
String header;

// Auxiliar variables to store the current output state
String entryState = "unknown";

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0;
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 2000;



void tick()
{
  //toggle state
  int state = digitalRead(BUILTIN_LED);  // get the current state of GPIO1 pin
  digitalWrite(BUILTIN_LED, !state);     // set pin to the opposite state
}

//gets called when WiFiManager enters configuration mode
void configModeCallback (WiFiManager *myWiFiManager) {
  //Serial.println("Entered config mode");
  //Serial.println(WiFi.softAPIP());
  //if you used auto generated SSID, print it
  //Serial.println(myWiFiManager->getConfigPortalSSID());
  //entered config mode, make led toggle faster
  ticker.attach(0.2, tick);
}

void setup() {
  // put your setup code here, to run once:
  //Serial.begin(115200);
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.setPixelColor(0, strip.Color(0, 0, 0));
  strip.show(); //turn off all LEDS
  //set led pin as output
  pinMode(BUILTIN_LED, OUTPUT);
  // start ticker with 0.5 because we start in AP mode and try to connect
  ticker.attach(0.6, tick);

  //WiFiManager
  //Local intialization. Once its business is done, there is no need to keep it around
  WiFiManager wifiManager;
  //reset settings - for testing
  //wifiManager.resetSettings();

  //set callback that gets called when connecting to previous WiFi fails, and enters Access Point mode
  wifiManager.setAPCallback(configModeCallback);

  //fetches ssid and pass and tries to connect
  //if it does not connect it starts an access point with the specified name
  //here  "AutoConnectAP"
  //and goes into a blocking loop awaiting configuration
  if (!wifiManager.autoConnect()) {
    //Serial.println("failed to connect and hit timeout");
    //reset and try again, or maybe put it to deep sleep
    ESP.reset();
    delay(1000);
  }

  //if you get here you have connected to the WiFi
  //Serial.println("connected...yeey :)");
  ticker.detach();
  //keep LED on
  digitalWrite(BUILTIN_LED, LOW);
  //Set RGB led to Blue until new data is sent
  strip.setPixelColor(0, strip.Color(0, 0, 250));
  strip.show();
  //strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)

  //Start LED Control Webserver
  server.begin();

}


//Loop for webUI
void loop() {
  WiFiClient client = server.available();   // Listen for incoming clients

  if (client) {                             // If a new client connects,
    //Serial.println("New Client.");          // print a message out in the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    currentTime = millis();
    previousTime = currentTime;
    while (client.connected() && currentTime - previousTime <= timeoutTime) { // loop while the client's connected
      currentTime = millis();
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        //Serial.write(c);                    // print it out the serial monitor
        header += c;
        if (c == '\n') {                    // if the byte is a newline character
          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            //Colors: (R,G,B)
            if (header.indexOf("GET /state/safe") >= 0) {
              entryState = "safe";
              strip.setPixelColor(0, strip.Color(0, 255, 0)); //GREEN
              strip.show();
            } else if (header.indexOf("GET /state/caution") >= 0) {
              entryState = "caution";
              strip.setPixelColor(0, strip.Color(150, 150, 0)); //YELLOW
              strip.show();
            } else if (header.indexOf("GET /state/unsafe") >= 0) {
              entryState = "unsafe";
              strip.setPixelColor(0, strip.Color(255, 0, 0)); //RED
              strip.show();
            }

            // Display the HTML web page
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // CSS to style the on/off buttons
            // Feel free to change the background-color and font-size attributes to fit your preferences
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".safebutton { background-color: #287a33; border: none; color: white; padding: 16px 20px;}");
            client.println(".cautionbutton { background-color: #d9b62b; border: none; color: white; padding: 16px 20px;}");
            client.println(".unsafebutton { background-color: #a31515; border: none; color: white; padding: 16px 20px;}");
            client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}</style>");
            
            // Web Page Heading
            client.println("<body><p>Lab Entry Indicator</p>");
            client.println("<h2 style='text-transform: uppercase;'>" + entryState + "</h2>");
            // Display current status
            if (entryState == "safe") {
              client.println("<h1 style='color: #287a33;'>It is safe to enter.</h1>");
            } else if (entryState == "caution") {
              client.println("<h1 style='color: #d9b62b;'>Entry is permitted, but use caution.</h1>");
            } else if (entryState == "unsafe") {
              client.println("<h1 style='color: #a31515;'>Entry is prohibited.</h1>");
            } else {
              client.println("<h1>Evaluate situation prior to entry.</h1>");
            }
            client.println("<p><a href=\"/state/safe\"><button class=\"safebutton safe\">Set as SAFE</button></a></p>");
            client.println("<p><a href=\"/state/caution\"><button class=\"cautionbutton caution\">Set as CAUTION</button></a></p>");
            client.println("<p><a href=\"/state/unsafe\"><button class=\"unsafebutton unsafe\">Set as UNSAFE</button></a></p>");

            client.println("</body></html>");

            // The HTTP response ends with another blank line
            client.println();
            // Break out of the while loop
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }
      }
    }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
  }
}
